=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2009-05-04 20:28:50 +0000
+++ loggerhead/apps/transport.py	2009-05-26 16:21:53 +0000
@@ -1,6 +1,7 @@
 """Serve branches at urls that mimic a transport's file system layout."""
 
 from bzrlib import branch, errors, lru_cache, urlutils
+from bzrlib.bzrdir import BzrDir
 
 from paste.request import path_info_pop
 from paste import httpexceptions
@@ -63,7 +64,10 @@
                 raise httpexceptions.HTTPNotFound()
             return self.app_for_non_branch(environ)(environ, start_response)
         else:
-            return self.app_for_branch(b)(environ, start_response)
+            if b.get_config().get_user_option('http_serve') == 'False':
+                raise httpexceptions.HTTPNotFound()
+            else:
+                return self.app_for_branch(b)(environ, start_response)
 
 
 class BranchesFromTransportRoot(object):
@@ -82,6 +86,14 @@
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
         elif '/.bzr/' in environ['PATH_INFO']:
+            try:
+                bzrdir = BzrDir.open_containing_from_transport(
+                         self.transport.clone(environ['PATH_INFO']))
+                branch = bzrdir.open_branch()
+                if branch.get_config().get_user_option('http_serve') == 'False':
+                    raise httpexceptions.HTTPNotFound()
+            except errors.NotBranchError:
+                pass
             app = urlparser.make_static(None, self.transport)
             return app(environ, start_response)
         else:

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2009-05-05 18:37:26 +0000
+++ loggerhead/controllers/directory_ui.py	2009-05-26 16:21:53 +0000
@@ -72,6 +72,8 @@
         for d in listing:
             try:
                 b = branch.Branch.open_from_transport(self.transport.clone(d))
+            	if b.get_config().get_user_option('http_serve') == 'False':
+                    continue
             except:
                 if not stat.S_ISDIR(self.transport.stat(d).st_mode):
                     continue

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2009-02-18 10:40:22 +0000
+++ loggerhead/tests/test_simple.py	2009-05-26 16:21:53 +0000
@@ -3,6 +3,7 @@
 
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.util.configobj.configobj import ConfigObj
+from bzrlib import config, osutils
 
 from loggerhead.apps.branch import BranchWSGIApp
 from paste.fixture import TestApp
@@ -118,3 +119,20 @@
         res = app.get('/files')
         res.mustcontain('No revisions!')
 
+
+class TestHiddenBranch(BasicTests):
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        locations = config.locations_config_filename()
+        config.ensure_config_dir_exists()
+        open(locations, 'wb').write('[%s]\nhttp_serve = False'
+                                    % (self.tree.branch.base,))
+
+    def test_no_access(self):
+        app = self.setUpLoggerhead()
+        print self.tree.branch.get_config().get_user_option('http_serve')
+        print app.get('/changes')
+        res = app.get('/changes', status=404)
+

